#!/usr/bin/env perl
# Zanata utility scripts
#
package Zanata::Util;
use version; our $VERSION = '0.2.9';
# Ensure it runs on RHEL5
use 5.008_008;
use strict;
use File::Find::Rule;
use File::Slurp;
use File::Spec;
use XML::Twig;

my $cfgFilename="zanata.cfg";
my @cfgFileSearchPaths=( "$ENV{'HOME'}/.config", '/etc', 'etc', '.');
my @argumentList=('url', 'project', 'project-version');

###########################################################
# String handling subroutine
#
sub trim {
    $_[0]=~s/^\s+//;
    $_[0]=~s/\s+$//;
}

sub unquote {
    if ( $_[0]=~m/^\'/ ){
	if ( $_[0]=~m/\'$/ ){
	    $_[0]=~ s/^\'//;
	    $_[0]=~ s/\'$//;
	}
    }elsif ( $_[0] =~ /^\"/ ){
	if ( $_[0] =~ /\"$/ ){
	    $_[0]=~ s/^\"//;
	    $_[0]=~ s/\"$//;
	}
    }
}

sub expand_string {
    trim($_[0]);
    unquote($_[0]);
}


###########################################################
# Config file handling subroutine
#

## hash_merge( templateHash_ref, supplementalHash_ref, mode )
sub hash_merge{
    my ($templateHash_ref, $supplementalHash_ref, $mode)=@_;
    # mode 0: supplemental override when value of corresponding key is empty
    # mode 1: supplemental override when
    if ($mode eq ""){
	$mode=0;
    }

    for my $key (keys %$templateHash_ref){
	if ($mode==0 and $templateHash_ref->{$key} ne ""){
	    next;
	}
	$templateHash_ref->{$key}=$supplementalHash_ref->{$key};
    }
}


=item find_file($file, @searchPaths)

Find file in search paths

=cut
sub find_file{
    my $file=shift;
    my @searchPaths=@_;

    my @files=File::Find::Rule->file()->name($file)->in(@searchPaths);
    for my $f (@files){
	if ( -r $f){
	    return $f;
	    last;
	}
    }
    die "File $file is not in either [" . join(", ", @searchPaths) . "]";
}

sub read_cfg_file{
    my ($isEnvOverride, $cfgFile)=@_;
    if ( $isEnvOverride== undef){
	$isEnvOverride=0;
    }

    $cfgFile=find_file($cfgFilename, @cfgFileSearchPaths) unless ($cfgFile);
    my %cfgVarH={};

    open(my $IN_FILE, "<$cfgFile") or die "Cannot open $cfgFile";
    while (my $line=<$IN_FILE>){
	next if $line =~ m/^\s*$/;
	next if $line =~  m/^\s*#/;
	chomp($line);
	my ($var, $val)=split(/=/,$line,2);
	expand_string($var);
	expand_string($val);
	if ($isEnvOverride and $ENV{$var} ne "" ){
	    # Env override mode
	    $cfgVarH{$var}=$ENV{$var};
	}else{
	    $cfgVarH{$var}=$val;
	}
    }
    close($IN_FILE);
    return \%cfgVarH;
}


###########################################################
# File handling subroutine
#

sub true_path{
    my ($baseDir, $path, $defaultPath)=@_;
    #print "$baseDir, $path, $defaultPath\n";
    ## Absolute path
    if ($path =~ '^/'){
	return $path;
    }
    return File::Spec->canonpath(($path) ?
	File::Spec->join($baseDir, $path): File::Spec->join($baseDir, $defaultPath));
}

sub replace_string_with_file{
    my ($inFile, $string, $snippetFile, $outFile) = @ARGV;
    my $outF;
    if ($outFile){
	open $outF, ">$outFile" or die $!;
    }else{
	$outF=*STDOUT;
    }
    $outF=*STDOUT unless($outFile);

    my $snippet=File::Slurp::read_file($snippetFile) or die "$!";

    open my $inF, "<$inFile" or die $!;

    while(<$inF>){
	if (/$string/){
	    print {$outF} $snippet;
	}else{
	    print {$outF} $_;
	}
    }
    close $inF;
    close $outF if $outFile;
}

###########################################################
# XML handling subroutine
#
## New xml_element from tag_name and content str
sub xml_element_new{
    my ($tagName, %optHash)= @_;
    my $new_element= XML::Twig::XPath::Elt->new($tagName);
    xml_element_set($new_element, %optHash);
    return $new_element;
}

sub xml_element_new_file{
    my ($tagName, $contentFile)= @_;
    return xml_element_new($tagName, content=>File::Slurp::read_file($contentFile));
}

## New xml_element from xml string
sub xml_element_new_xml{
    my ($xmlStr)= @_;
    my $new_element= XML::Twig::XPath::Elt->parse($xmlStr);
    return $new_element;
}

sub xml_element_new_xml_file{
    my ($xmlFile)= @_;
    my $parser= XML::Twig::XPath->new( pretty_print=>'indented');
    my $doc=$parser->parsefile("$xmlFile");
    return $doc->root;
}

=item xml_element_set(elt, opt_hash)

Set content and attributes of an xml elements.

elt is element to be set.

opt_hash is a hash that contains following keys:

=over 4

=item tag_name

Tag name to be set.

=item content

Content to be set.

=item attributes

Hash of attributes

=back

=cut

sub xml_element_set{
    my ($elt, %opt_hash)= @_;

    if (exists $opt_hash{'tag_name'}){
	$elt->set_gi($opt_hash{'tag_name'});
    }

    $elt->cut_children();

    if (exists $opt_hash{'content'}){
	$elt->set_text($opt_hash{'content'});
    }

    if (exists $opt_hash{'attributes'}){
	while( my ($att, $val)= each (%{$opt_hash{'attributes'}})){
	    $elt->set_att( $att => $val);
	}
    }
    return $elt;
}

=item xml_element_ensure_child(parent, elt_name, opt_hash)

Ensure an xml element has a child.
If no such child exists, then create one.

Arguments:

=over 4

=item parent

Parent element

=item elt_name

Tag name of child element

=item opt_hash

Optional options pass as hash.

Optional options are:

=over 4

=item content

Content to be ensured.

=item attributes

Hash of attributes

= item

=item is_replace

Whether to replace value of existing child.

=item position

Position to insert created element. This is passed to XML::Twig::Elt::insert_new_element.

Valid values:

=over 4

=item first_child (default)

The element is pasted as the first child of $ref

=item last_child

The element is pasted as the last child of $ref

=item before

The element is pasted before $ref, as its previous sibling.

=item after

The element is pasted after $ref, as its next sibling.

=item within
In this case an extra argument, $offset, should be supplied. The element will be pasted in the reference element (or in its first text child) at the given offset. To achieve this the reference element will be split at the offset.

=back

=back

=back

Returns the child element.

=cut

sub xml_element_ensure_child{
    my ($parent, $elt_name, %opt_hash)= @_;
    my $xpath=$elt_name;

    my ($elt)=$parent->findnodes($xpath);
    ## Unless child exists
    my $position= ($opt_hash{'position'}) ? $opt_hash{'position'}: 'first_child';
    unless($elt){
	### Insert new element
	$elt=$parent->insert_new_elt($position, $elt_name);
	xml_element_set($elt, %opt_hash);
    }elsif ($opt_hash{'is_replace'}){
	xml_element_set($elt, %opt_hash);
    }
    return $elt;
}


=item build_xpath

This function check whether the elements in xpath exists,
and create the necessary elements in the path.

Arguments:

=over 4

=item twig

Twig to be processed.

=item xpath

Xpath to be built.

=item opt_position

Position for the created element. This is passed to XML::Twig::Elt::insert_new_element.

Valid values:

=over 4

=item first_child (default)

The element is pasted as the first child of $ref

=item last_child

The element is pasted as the last child of $ref

=item before

The element is pasted before $ref, as its previous sibling.

=item after

The element is pasted after $ref, as its next sibling.

=item opt_position

Position to insert created element.
See opt_position in L</"xml_element_ensure_child"> for details.

=item within
In this case an extra argument, $offset, should be supplied. The element will be pasted in the reference element (or in its first text child) at the given offset. To achieve this the reference element will be split at the offset.

=back

=item opt_offset

This argument is only used if opt_position is 'within'

=back

Note this only handle simple cases like absolute file path (e.g. /ppp/qqq)

Return: the deepest element.

=cut

sub build_xpath{
    my ($twig, $xpath, $opt_position, $offset)= @_;
    $opt_position='first_child' unless ($opt_position);
    my @paths=split /\//, $xpath;
    shift @paths;
    my $rootP= shift @paths;

    my $root=$twig->root;
    unless($root){
	my $newElt=XML::Twig::XPath::Elt->new($rootP);
	$twig->set_root($newElt);
	$root=$twig->root;
    }
    my $parent=$root;
    while(my $p=shift @paths){
	my $elt=xml_element_ensure_child($parent, $p, '',  0, $opt_position);
	$parent=$elt;
    }
    return $parent;
}

###########################################################
# Misc subroutines
#


=item init($cfgFile)

Initial this util by reading the cfg file and set server info.

B<Arguments>
=over 4

=item cfgFile

Optional cfgFile

=back

=cut
sub init{
    my $class=shift;
    my ($projInfoRef, $cfgFile)=@_;
    my $url=$projInfoRef->{'url'};
    #print "url=" . $projInfoRef->{'url'} ."\n";

    my $self={
	## Hash that holds variable defined in cfg file.
	cfgVarH=>{}

	, serverVer => undef

	### URL for the Zanata server home page.
	, serverUrl=> $url

	### URL for actually download the zanata.xml
	, zanataXmlUrl => undef

	### URL to downoload the language list.
	, languageListUrl => ($url) ? "${url}language/list" : undef
    };

    $self->{'cfgVarH'}=read_cfg_file(1, $cfgFile);

    if ($url){
	## Set zanata.xml download url
	if ($projInfoRef->{'project'} and $projInfoRef->{'project-version'}){
	    my $zanataXml=$self->{cfgVarH}->{ZANATA_XML_URL};
	    for my $v (@argumentList){
		$zanataXml =~ s|\@$v\@|$projInfoRef->{$v}|;
	    }
	    $self->{'zanataXmlUrl'}=$zanataXml;
	}
    }

    bless ($self,$class);
    return $self;
}

1;

