#!/usr/bin/env perl
# Package that deal with pom.xml
#
package Zanata::Util::PomXML;
use 5.008_008;
use strict;
use File::Basename;
use File::Slurp qw(read_file);
use File::Spec;
use LWP::UserAgent;
use XML::Twig::XPath;

use constant PluginsXPath => '/project/build//plugins';
use constant ZanataPluginXPath => PluginsXPath . '/plugin[groupId=\'org.zanata\']';

# Element hash
my %eltHash=(
	properties => '/project/properties'
	, plugins => PluginsXPath
	, zanataPlugin=> ZanataPluginXPath
	, configuraton =>ZanataPluginXPath . '/configuration'
    );
my @propertyList= qw{ zanataMavenPluginVer };


=item new(content=>I<str>, zanataPluginVersion=>I<str> )

New a PomXML instance. If content is given, the value is set accordingly.

B<Options>

=over 4

=item content

A string representation of xml.

=item zanataPluginVersion

A string that specify the zanata plugin version

=back


B<Properties>

=over 4

=item elementHash

Hash that hold various element references

=item plugins

Reference to /project/build/plugins

=back

=cut
sub new{
    my ($class, %optHash)=@_;
    my $twig=XML::Twig::XPath->new(pretty_print=>'indented');

    my $self={};
    bless ($self,$class);
    $self->{'twig'}=$twig;
    $self->{'elementHash'}={};

    if ($optHash{'content'}){
	$self->parse($optHash{'content'});
    }else{
	my $rElt=XML::Twig::XPath::Elt->new('project');
	$twig->set_root($rElt);
	$twig->set_doctype(0,0,0,0);
    }

    $self->fix(\%optHash);
    return $self;
}

# Fix the pom.xml by adding the missing part, updating the version
sub fix{
    my ($self, $optHashRef)=@_;
    ## Set root
    my $twig=$self->{'twig'};
    my $root=$twig->root;

    ## Complete top level properties.
    ## Note that we should not modify the top level properties if either
    ## 1) groupId exists and it is not "org.zanata"
    ## 2) artifactId exists and it is not "null"

    unless($root->first_child('artifactId') and $root->first_child('artifactId')->text ne 'null'){
	unless($root->first_child('groupId') and $root->first_child('groupId')->text ne 'org.zanata'){
	    Zanata::Util::xml_element_ensure_child($root, 'version',
		content=> ($optHashRef->{'zanataPluginVersion'}), is_replace=>1);
	    Zanata::Util::xml_element_ensure_child($root, 'modelVersion',
		content=> '4.0.0', is_replace=>1);
	    Zanata::Util::xml_element_ensure_child($root, 'artifactId',
		content=> 'null', is_replace=>1);
	    Zanata::Util::xml_element_ensure_child($root, 'groupId',
		content=> 'org.zanata', is_replace=>1);
	}
    }

    unless($self->{'elementHash'}->{'properties'}){
	$self->{'elementHash'}->{'properties'}=Zanata::Util::xml_element_ensure_child($root, 'properties');
    }

    # Add missing zanataPlugin
    my $zanataPluginElt=$self->{'elementHash'}->{'zanataPlugin'};
    unless($zanataPluginElt){
	my $pluginsElt=$self->{'elementHash'}->{'plugins'};
	unless ($pluginsElt){
	    $pluginsElt=Zanata::Util::build_xpath($twig, '/project/build/plugins');
	    $self->{'elementHash'}->{'plugins'}=$pluginsElt;
	}

	$zanataPluginElt= $pluginsElt->insert_new_elt('plugin');
	$zanataPluginElt->insert_new_elt(groupId => "org.zanata");
	$self->{'elementHash'}->{'zanataPlugin'}=$zanataPluginElt;
    }

    Zanata::Util::xml_element_ensure_child($zanataPluginElt, 'artifactId',
	content=> 'zanata-maven-plugin', is_replace=>1, position=> 'last_child');
    Zanata::Util::xml_element_ensure_child($zanataPluginElt, 'version',
	content=> $optHashRef->{'zanataPluginVersion'}, is_replace=>1, position=> 'last_child');

    $self->fix_configuration();
}

# This assume the main body is fixed.
# Move configuration to properties, so it can be overridden by command line options
sub fix_configuration{
    my ($self)=@_;
    my $propertiesElt=$self->{'elementHash'}->{'properties'};
    my $configurationElt=Zanata::Util::xml_element_ensure_child($self->{'elementHash'}->{'zanataPlugin'}, 'configuration');
    $self->{'elementHash'}->{'configuration'}=$configurationElt;
    foreach my $elt ($configurationElt->children()){
	# print STDERR "elt=<" . $elt->gi. ">" .$elt->text ."</". $elt->gi.">\n";
	$self->configuration_move_to_properties($elt->gi);
    }
}

# Move a configuration option to properties, to make it overridable from command line option
sub configuration_move_to_properties{
    my ($self, $tagName)=@_;
    my $propertiesElt=$self->{'elementHash'}->{'properties'};
    my $configurationElt=$self->{'elementHash'}->{'configuration'};
    my $propTagName="zanata.$tagName";

    ## Rename and move the elt to properties unless  propTag exists in properties (i.e. it has been done previously(
    unless($propertiesElt->first_child($propTagName)){
	my $elt=$configurationElt->first_child($tagName);
	$elt->set_gi($propTagName);
	$elt->move(last_child => $propertiesElt);
	Zanata::Util::xml_element_ensure_child($configurationElt, $tagName,
	    content=> '${' . $propTagName .'}', is_replace=>1, position=> 'last_child');
	return $elt
    }

    return undef;
}

sub parse{
    my ($self, $content)=@_;
    my $twig=$self->{'twig'};

    ## Return undef if failed on safe_parse
    return undef unless ($twig->safe_parse($content));

    ## Return undef if the content is not pom.xml
    return undef unless &is_pom_xml($twig);

    ## Filling the references by parsing xml

    while ( my ($p, $xpath) = each(%eltHash)){
	# print "p=$p xpath=$xpath\n";
	my ($elt)=$twig->findnodes($xpath);
	if ($elt){
	    $self->{'elementHash'}->{$p}=$elt;
	}
    }
}

sub is_pom_xml{
    my ($twig)=@_;
    if ($twig->root->tag eq "project"){
	return 1;
    }
    return 0;
}


=item add_configuration(tagName, optHash)

Add a new element in configuration.

=cut

sub add_configuration{
    my ($self, $tagName, %optHash)=@_;
    my $configurationElt=$self->{'elementHash'}->{'configuration'};

    # Do not touch the existing configuration
    my $elt=undef;
    unless($configurationElt->first_child($tagName)){
	$elt=Zanata::Util::xml_element_new($tagName, %optHash);
	$elt->paste(last_child=>$configurationElt);
    }

    return $elt;
}

=item add_configuration_overridable(tagName, optHash)

Add an element to configuration section, yet make it overridable from command line.
Note that if

=cut

sub add_configuration_overridable{
    my ($self, $tagName, %optHash)=@_;
    my $elt=$self->add_configuration($tagName, %optHash);
    if ($elt){
	$self->configuration_move_to_properties($tagName);
    }
    return $elt;
}


sub to_xml{
    my ($self, $outputFilename)=@_;
    my $twig=$self->{'twig'};

    if ($outputFilename){
	$twig->print_to_file($outputFilename) or die "Cannot write to $outputFilename: $!";
    }
    return $twig;
}

1;
