#!/usr/bin/env perl
# Package that deal with zanata.xml
#
package Zanata::Util::ZanataXML;
use 5.008_008;
use strict;
use File::Basename;
use File::Slurp qw(read_file);
use File::Spec;
use LWP::UserAgent;
use XML::Twig;

use constant ZanataNameSpace => 'http://zanata.org/namespace/config/';
my @fieldList= qw{ url project project-version project-type };


=item new($content)

New a ZanataXML instance. If content is given, the value is set accordingly.

B<Parameters>

=over 4

=item content

A string representation of xml.

=back

B<Properties>

=over 4

=item url

URL to zanata server

=item project

Project ID.

=item project-version

Project version.

=item project-type

Project type.

=item locales

Hash of locales. Locale acts as key while either  map-from (when it has map-from) or 1 as value.

=back

=cut
sub new{
    my $class=shift;
    my $content= shift;
    my $twig=XML::Twig->new(pretty_print=>'indented');

    my $self={};
    for my $f (@fieldList){
	$self->{$f}=undef;
    }
    $self->{'locales'}={};

    bless ($self,$class);
    if ($content){
	return undef unless $self->parse($content);
    }
    return $self;
}

sub parse{
    my ($self, $content)=@_;
    my $twig=XML::Twig->new(pretty_print=>'indented');

    ## Return undef if failed on safe_parse
    unless($twig->safe_parse($content)){
	warn "[Warn] ZanataXML:parse(): Cannot safe_parse the content\n";
	return undef;
    }

    ## Return undef if the content is not zanata.xml
    unless(&is_zanata_xml($twig)){
	warn "[Warn] ZanataXML:parse(): The content is not zanata.xml\n";
	return undef;
    }

    ## Filling the fields by parsing xml
    for my $f (@fieldList){
	my $elt=$twig->root->first_child($f);
	if ($elt){
	    $self->{$f}=$elt->trimmed_text;
	}
    }
    ## Read locales
    $self->parse_locales( $twig);
    $twig->purge;
    return 1;
}


=item new_from_url($url)

New a ZanataXML instance from a url

=over 4

=item url

url is the actual url to access zanata.xml, not the zanata server URL.

=back

=cut

sub new_from_url{
    my $class=shift;
    my $url= shift;
    my $ua = LWP::UserAgent->new;
    $ua->timeout(10);
    $ua->env_proxy;

    my $response = $ua->get($url);
    unless ($response->is_success) {
	warn "[Warn] Failed to download zanata.xml from $url\n";
	warn "[Warn] $response->status_line .\n";
	return undef;
    }
    return Zanata::Util::ZanataXML->new($response->decoded_content);
}

=item get_locales()
Get the list of locales.

B<Returns:> list of locales.
=cut
sub get_locales{
    my ($self)=@_;
    return keys %{$self->{'locales'}};
}

=item update_locale($locale, $mapFrom)

Insert or update locale and its mapFrom.

=over 4

=item locale

Locale that enabled in Zanata

=item mapFrom

For attribute "mapFrom" for tag E<lt>localeE<gt>.
The attribute will be omitted if mapFrom is either empty or 1.

=back

=cut
sub update_locale{
    my ($self, $locale, $mapFrom)=@_;
    $mapFrom=1 unless $mapFrom;
    $self->{'locales'}->{$locale}=$mapFrom;
}

=item parse_locales($twig)

Parse locales and map-from from XML

B<Returns:> locales hash which locale is key and either map-from (when it has map-from) or 1 as value.
Or undef if xml does not contain any locale.

=cut
sub parse_locales{
    my ($self, $twig)=@_;

    # Quit if xml does not contain locales
    my $localesElt=$twig->root->first_child('locales');
    return undef unless $localesElt;
    return undef unless $localesElt->first_child('locale');

    my @localeEltL=$localesElt->children('locale');
    for my $localeElt (@localeEltL){
	# Insert locale and map-from to localeH
	my $key=$localeElt->trimmed_text;
	next unless $key;
	my $att=$localeElt->att('map-from');
	$self->update_locale($key, $att);
    }
    return $self->{'locales'};
}


sub is_zanata_xml{
    my ($twig)=@_;
    if ($twig->root->tag eq "config"){
	if ($twig->root->att('xmlns') eq ZanataNameSpace){
	    return 1;
	}
    }
    return 0;
}

sub to_xml{
    my ($self, $outputFilename)=@_;
    my $twig=XML::Twig->new(pretty_print=>'indented');

    ## Set root
    my $rElt=XML::Twig::XPath::Elt->new('config');
    $twig->set_root($rElt);
    $twig->root->set_att(xmlns=>ZanataNameSpace);
    $twig->set_doctype(0,0,0,0);

    my $root=$twig->root;
    ## Set the fields except locales
    for my $f (@fieldList){
	if ($self->{$f}){
	    $root->insert_new_elt('last_child', $f => $self->{$f});
	}
    }

    ## Set the locales
    my %localeH=%{$self->{'locales'}};
    if (keys %localeH){
	my $localesElt=$root->insert_new_elt('last_child', 'locales');
	for my $l (sort keys %localeH){
	    my $localeElt=$localesElt->insert_new_elt('last_child', locale=> $l);
	    my $mapFrom=$localeH{$l};
	    $localeElt->set_att('map-from', $mapFrom) unless $mapFrom eq "1";
	}
    }

    if ($outputFilename){
	$twig->print_to_file($outputFilename) or die "Cannot write to $outputFilename: $!";
    }
    $twig->purge;
    return $twig;
}

1;
